<?php

namespace Util;

use Teknisa\GER\Utilities\Environment;
use Teknisa\Libs\Util\SMTP;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class EmailController
{
    protected $connection;
    protected $fileServerURL;
    protected $smtpUtil;
    protected $crudService;
    private $isUsingCustomSmtp;
    
    public function __construct($connection, $environment, $fileServerURL, $smtpUtil, $crudService) 
    {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->fileServerURL = $fileServerURL;
        $this->crudService = $crudService;
        $this->templatesDir = realpath(__DIR__ . '/Template');
    }
    
    /**
     * Retorna um Cookie pelo nome
     */
    public function getCookieEmail($name) 
    {
        if(array_key_exists($name, $_COOKIE)){
            return $_COOKIE[$name];
        }
        
        return false;
    }
    
    public function getLgpd($cdFilial)
    {
        $sql = "SELECT 
                    P.TXAVISOLEGALEMAIL 
                FROM 
                    PARASUPRI P, 
                    FILIAL F 
                WHERE 
                    F.CDFILIAL = :P_CDFILIAL AND 
                    P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
                    
        $stmt = $this->connection->prepare($sql);
        
        $stmt->execute(
            array(
                "P_CDFILIAL" => $cdFilial
            )
        );
        
		$result = $stmt->fetchAll();
		
        return $result[0]['TXAVISOLEGALEMAIL'];
    }
    
    public function getLogo($idAplicativo = 'PORTAL E-COMMERCE') 
    {
        $sql = "SELECT 
                    URLLOGO 
                FROM 
                    APPCONFIG 
                WHERE 
                    IDAPLICATIVO = :P_IDAPLICATIVO AND 
                    NRORG = :P_NRORG";
                    
        $stmt = $this->connection->prepare($sql);
        
        $stmt->execute(
            array(
                "P_IDAPLICATIVO" => $idAplicativo,
                "P_NRORG" => $this->environment->getNrorgTrab()
            )
        );
        
		$result = $stmt->fetchAll();
		
		if (isset($result[0]) && $result[0]['URLLOGO'] != null) {
    		return $this->fileServerURL . $result[0]['URLLOGO'];
        } else {
            return false;
        }
    }
    
    public function getLogoAttributes(&$logoSrc, &$attrImg = 'width="200px"')
    {
        if ($logoSrc != false && $this->checkFileExists($logoSrc)) {
            $imgAttributes = getimagesize($logoSrc);
            
            if ($imgAttributes[1] > $imgAttributes[0]) {
                $attrImg = 'height="175px"';
            }
        } else {
            $logoSrc = 'https://www.teknisa.com/imagens/logo-teknisa.png';
        }
    }
    
    private function checkFileExists($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if ($code == 200) {
            $status = true;
        } else {
            $status = false;
        }
        
        curl_close($ch);
        return $status;
    }
    
    private function getFileContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    
    public function getEmailsRelacionadosFornecedor($fornecedores)
    {
        $listaEmails = array();
        
        if (is_array($fornecedores)) {
            $fornecedores = array_unique($fornecedores);
            
            foreach ($fornecedores as $fornecedor) {
                $listaEmails = $this->getListaEmails($fornecedor, $listaEmails);
            }
        } else {
            $listaEmails = $this->getListaEmails($fornecedores, $listaEmails);
        }
        
        if (!count($listaEmails)) {
            throw new \Exception("Expiração alterada. <br/> E-mail não enviado pois o fornecedor não possui acesso as cotações.");
        }
        
        $listaEmails = array_unique($listaEmails);
        
        return $listaEmails;
    }
    
    private function getListaEmails($cdForneced, &$listaEmails) {
        $emailFornecedores = $this->getEmailsRelacionadosByCdForneced($cdForneced)->getRows()[0];
            
        if ($emailFornecedores['DSLOGINECO']) {
            $listaEmails = array_merge($listaEmails, explode(',', $emailFornecedores['DSLOGINECO']));
            
            if ($emailFornecedores['DSEMAILECO']) {
                $listaEmails = array_merge($listaEmails, explode(',', $emailFornecedores['DSEMAILECO']));
            }
        }
        
        return $listaEmails;
    }
    
    private function getEmailsRelacionadosByCdForneced($cdforneced)
    {
        $conditions = [
            array(
                "columnName" => "CDFORNECED",
                "operator"   => '=',
                "value"      => $cdforneced
            )];

        return $this->crudService->getFilteredDataSet('emailsRelacionadosFornecedor', $conditions);
    }
    
    public function replaceMessageContent($logoSrc, $assunto, $message, $attrImg, $lgpd){
        $search = array(
            '%attrImg%',
            '%logoSrc%',
            '%assunto%',
            '%mensagem%',
            '%lgpd%'
        );
        
        $replace = array(
            $attrImg,
            $logoSrc,
            $assunto,
            $message,
            $lgpd
        );
            
        $subject = $this->getFileContentUtf8($this->templatesDir . "/email.html");
        
        return str_replace($search, $replace, $subject);
    }
    
    public function addAttachments($attachments, $clearAttachments = false)
    {
        if ($clearAttachments) {
            $this->smtpUtil->clearAttachments();
        }

        foreach ($attachments as $attachment) {
            $this->smtpUtil->addAttachment(
                $attachment->getFile(),
                $attachment->getName(),
                $attachment->getEncoding(),
                $attachment->getType(),
                $attachment->getDisposition(),
                $attachment->isStringAttachment()
            );
        }
    }
    
    public function enviaEmail($subject, $message, $emails) {
        $this->useCustomSmtp();
        $this->smtpUtil->setSubject($subject);
        $this->smtpUtil->sendMail($message, $emails);
    }
    
    /*********************************************************************************************/
    /**************************************** CUSTOM SMTP ****************************************/
    /*********************************************************************************************/
    
    public function useCustomSmtp()
    {
        $params = $this->getCustomSmtpSettings();

        $this->smtpUtil = new SMTP($params);

        if (isset($params['fromName'])) {
            $this->smtpUtil->setFromName($params['fromName']);
        }

        if (isset($params['from'])) {
            $this->smtpUtil->setNameReplyTo($params['from']);
        }

        $this->isUsingCustomSmtp = true;
    }

    public function getCustomSmtpSettings()
    {
        $tipoDadoUtilizado = $this->getTipoDadoUilizadoEnvioEmail()['IDUTDADOPERENVEMAIL'];
        $dadosSMTPFilial   = $this->getDadosSMTPFromFilial();
        $settings          = self::getSmtpSettingsFromAddress($dadosSMTPFilial['DSSMTPEMAILF']);
        
        $params['host'] = $settings['host'];
        
        if (isset($settings['port'])) {
            $params['port'] = $settings['port'];
        }

        if ($dadosSMTPFilial['IDUTTLSEMAIL'] == 'S') {
            $params['smtpSecure'] = 'tls';
        } else if ($dadosSMTPFilial['IDUTSSLEMAIL'] == 'S') {
            $params['smtpSecure'] = 'ssl';
        }

        switch($tipoDadoUtilizado) {
            case 'LOGIN_FILIAL_REMETENTE_FILIAL':
                $params['from']     = $dadosSMTPFilial['DSEMAILFILI'];
                $params['fromName'] = $dadosSMTPFilial['DSEMAILFILI'];
                $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                $params['password'] = $dadosSMTPFilial['CDSENHEMAILF'];
            break;
             
            case 'LOGIN_FILIAL_REMETENTE_OPER':
                $dadosEmailOperadorLogado = $this->getDadosEmailOperadorLogado();
                
                $params['from']     = $dadosEmailOperadorLogado['DSEMAILOPER'];
                $params['fromName'] = $dadosEmailOperadorLogado['NMOPERADOR'];
                $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                $params['password'] = $dadosSMTPFilial['CDSENHEMAILF'];
            break;
            
            case 'LOGIN_OPER_REMETENTE_OPER':
                $dadosEmailOperadorLogado = $this->getDadosEmailOperadorLogado();

                $params['from']     = $dadosEmailOperadorLogado['DSEMAILOPER'];
                $params['fromName'] = $dadosEmailOperadorLogado['NMOPERADOR'];
                $params['username'] = $dadosEmailOperadorLogado['DSEMAILOPER'];
                $params['password'] = $dadosEmailOperadorLogado['CDSENEMAILOP'];
            break;
        }
        
        if (isset($params['password'])) {
            $params['password'] = self::getDelphiPasswordEncrypted($params['password'], false);
        }
        
        return $params;
    }
    
    /**
     * Recupera informações do endereço de SMTP, como:
     * - host;
     * - porta;
     * 
     * @param string $url Url do servidor com ou sem porta.
     * 
     * @return array Contendo as informações encontradas.
     */
    public function getSmtpSettingsFromAddress($url)
    {
        $match = self::splitByCommonDelimiters($url);
        
        if (empty($match)) {
            throw new \Exception('Formato do endereço de SMTP inválido.');
        }
        
        if (count($match) > 1) {
            $params['port'] = $match[1];
        }
        
        $params['host'] = $match[0];
        
        return $params;
    }
    
    public function splitByCommonDelimiters($string)
    {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);
        
        $splittedString = preg_split($pattern, $cleanedString);
        
        return $splittedString;
    }
    
    public function removeAllWhitespaces($string)
    {
        $pattern = '/\s+/';
        
        $cleanedString = preg_replace($pattern, '', $string);
        
        return $cleanedString;
    }
    
    public function getTipoDadoUilizadoEnvioEmail()
    {
        $query = "SELECT 
                      PARASUPRI.IDUTDADOPERENVEMAIL
                  FROM 
                      PARASUPRI
                  WHERE 
                      PARASUPRI.CDFILIAL = :P_CDFILCENTCMP";
                      
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILCENTCMP" => self::getCentralCompras($this->environment->getCdFilial())['CDFILCENTCMP'],
        ));
        
        return $std->fetch();
    }
    
    public function getCentralCompras($codigoFilial)
    {
        $query = "SELECT
                      F.CDFILIAL,
                      F.NMFILIAL,
                      F2.CDFILIAL AS CDFILCENTCMP,
                      F2.NMFILIAL AS NMFILCENTCMP,
                      F2.NRORG
                  FROM
                      FILIAL F,
                      FILIAL F2
                  WHERE
                      F.CDFILIAL = :CDFILIAL AND 
                      F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
                      
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "CDFILIAL" => $codigoFilial,
        ));
        
        return $std->fetch();
    }
    
    public function getDadosSMTPFromFilial()
    {
         $query = " SELECT 
                        EF.CDSENHEMAILF, 
                        EF.DSSMTPEMAILF, 
                        NVL(EF.DSLOGINSMTP, EF.DSEMAILFILI) DSLOGINSMTP, 
                        NVL(EF.DSEMAILFILI, EF.DSLOGINSMTP) DSEMAILFILI, 
                        EF.IDUTTLSEMAIL, 
                        EF.IDUTSSLEMAIL, 
                        FI.CDFILIAL || ' - ' || FI.NMFILIAL AS NMFILIAL
                    FROM 
                        ENDEFILI EF 
                        INNER JOIN FILIAL FI ON EF.CDFILIAL = FI.CDFILIAL
                    WHERE 
                        EF.CDFILIAL = :P_CDFILIPROC
                        AND EF.IDTPENDEFILI = 'P'";
                      
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILIPROC" => $this->environment->getCdFilial(),
        ));
        
        return $std->fetch();
    }
    
    public function getDadosEmailOperadorLogado()
    {
        $query = "SELECT 
                      OP.NMOPERADOR, 
                      OP.DSEMAILOPER, 
                      OP.CDSENEMAILOP
                  FROM 
                      OPERADOR OP
                  WHERE 
                      OP.CDOPERADOR = :P_CDOPERADOR";
                      
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDOPERADOR" => $this->environment->getCdOperador(),
        ));
        
        return $std->fetch();
    }
    
    /**
     * Temporário até que uma solução seja implementada no local original desta
     * função.
     * 
     * A original realiza chamada apenas nas funções GetCriptoPass e GetUnCriptoPass,
     * equivalentes ao CriptoPlus e UncriptoPlus do Delphi.
     * 
     * Porém, utilizamos as criptografias convencionais, que até então não eram
     * implementadas la Bib_html.
     * 
     * Foi incluída na 'Bib', rdm 'rdmDBGeral', na issue #238324
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
    
        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );
    
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
    
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
    
        return $encryption;
    }
    
    /**
     * Temporário até que uma solução seja implementada para a função getDelphiPasswordEncrypted.
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
} 